const pattern = /(.+)\/.+$/;
export default class RouteConfigurator {
    static resolveComponents(config) {
        return "component" in config
            ? new Map([["default", config.component]])
            : new Map(Object.entries(config.components));
    }
    static *configure(configurations, parent) {
        for (const config of configurations) {
            let hasDefaultChildren = false;
            const path = !parent || config.path.startsWith("/")
                ? config.path
                : config.path
                    ? `${parent.path}/${config.path}`
                    : parent.path;
            const components = RouteConfigurator.resolveComponents(config);
            if (components.size == 0) {
                throw new Error(`Route "${config.path}" requires at least one component`);
            }
            const definition = {
                constraints: { ...parent?.constraints, ...config.constraints },
                meta: config.meta ?? {},
                name: config.name,
                path,
                selector: config.selector,
                stack: [...parent?.stack ?? [], components],
                transformers: { ...parent?.transformers, ...config.transformers },
            };
            if ((config.children?.length ?? 0) > 0) {
                for (const child of RouteConfigurator.configure(config.children, definition)) {
                    if (child.path == definition.path) {
                        hasDefaultChildren = true;
                    }
                    yield child;
                }
            }
            if (hasDefaultChildren) {
                definition.path = definition.path.replace(pattern, "$1");
            }
            yield definition;
        }
    }
}
